
import * as server from "@minecraft/server"
import * as ui from '@minecraft/server-ui'
import * as gametest from '@minecraft/server-gametest'

import { forceShow ,getScore} from "./forceshow"



const CORPSE_NAME_PREFIX = `Corpse Of `
const NOTIFICATION_PREFIX = `§5[§dCorpses Addon§5]§e`
const testCorpseCommand = `cp! testCorpse `
const corpseDespawnCommand = `cp! doCorpseDespawn `
const noOpMessage = `§cYou dont have permisssion to use this`
const dimensions = [`overworld`,`nether`,`the_end`]

const test = gametest.Test
const world = server.world
const system = server.system

function corpseTransferItems(player,corpse){
    const {dimension,location} = player
    const slots = ["offhand","mainhand","head","chest","legs","feet"]
    const pInventory = player.getComponent(`inventory`).container
    const pEquipment = player.getComponent("equipment_inventory")
    const tInventory = corpse.getComponent(`inventory`).container
    const tEquipment = corpse.getComponent("equipment_inventory")
    const air =  new server.ItemStack("air")

    slots.forEach(slot=>{
        let tItem = tEquipment.getEquipment(slot)
        let pItem = pEquipment.getEquipment(slot)
        if(tItem){
            if(!pItem){
            
                tEquipment.setEquipment(slot,air)
                pEquipment.setEquipment(slot,tItem)
                
            }else{
                let residue = pInventory.addItem(tItem)
                if(residue){dimension.spawnItem(residue,location)}
                
            }
        }
    })

    for (let i = 0; i < tInventory.size; i++) {
        let item = tInventory.getItem(i)
        
        if(item){
          tInventory.setItem(i,air)
          let residue = pInventory.addItem(item)
          if(residue){dimension.spawnItem(residue,location)}
        }
    }

    
        corpse.triggerEvent(`despawn`)
    
    //corpse.triggerEvent(`despawn`)
}

world.afterEvents.worldInitialize.subscribe((data)=>{

    try{
        world.scoreboard.addObjective(`corpseRemainingDespawnTime`,`Corpse Despawn Time`)
      }catch{
        console.warn(``)
      }
      

    let corpseAddonSettings = new server.DynamicPropertiesDefinition()
    //let corpseData = new server.DynamicPropertiesDefinition()

    corpseAddonSettings.defineNumber(`corpseReviveChance`,0)
    corpseAddonSettings.defineNumber(`corpseDespawnTime`,300)
    corpseAddonSettings.defineBoolean(`corpseShowDeathCordinates`,true)
    corpseAddonSettings.defineBoolean(`corpseAnyPlayerCanAccess`,false)
    corpseAddonSettings.defineBoolean(`corpseKeepXpOnDeath`,false)
    corpseAddonSettings.defineBoolean(`corpseAlwaysShowNameTag`,true)
    corpseAddonSettings.defineBoolean(`corpseTellWhenDespawned`,true)
    corpseAddonSettings.defineBoolean(`corpseFloatsOnLiquids`,true)
    corpseAddonSettings.defineBoolean(`doCorpseDespawn`,true)

    //corpseData.defineNumber(`corpseRemainingDespawnTime`,-1)

    data.propertyRegistry.registerWorldDynamicProperties(corpseAddonSettings)
    //data.propertyRegistry.registerEntityTypeDynamicProperties(corpseData, `ax:tumba`)

   
})

world.afterEvents.chatSend.subscribe((data) =>{
    const {sender, message} = data
    
    if (message == `cp! settings`){
            
        if(sender.isOp()){
            const corpseShowDeathCordinates = world.getDynamicProperty(`corpseShowDeathCordinates`)
            const corpseAnyPlayerCanAccess = world.getDynamicProperty(`corpseAnyPlayerCanAccess`)
            const corpseAlwaysShowNameTag = world.getDynamicProperty(`corpseAlwaysShowNameTag`)
            const corpseFloatsOnLiquids = world.getDynamicProperty(`corpseFloatsOnLiquids`)
            const corpseKeepXpOnDeath = world.getDynamicProperty(`corpseKeepXpOnDeath`)
            const corpseReviveChance = world.getDynamicProperty(`corpseReviveChance`)
            const corpseDespawnTime = world.getDynamicProperty(`corpseDespawnTime`)
            const corpseTellWhenDespawned = world.getDynamicProperty(`corpseTellWhenDespawned`)

            const form = new ui.ModalFormData()

            form.title(`${NOTIFICATION_PREFIX} Settings`)
            
            form.toggle(`Show death cordinates to the dead player`,corpseShowDeathCordinates)
            form.toggle(`Players can break other player's corpses`,corpseAnyPlayerCanAccess)
            form.toggle(`Always show the corpse's Nametag`,corpseAlwaysShowNameTag)
            form.toggle(`Corpses can float on liquids`,corpseFloatsOnLiquids)
            form.toggle(`Players keep XP after death`,corpseKeepXpOnDeath)
            form.slider(`Percent Chance for corpses to become hostile\n(Set at 0 to turn off)`,0,100,1,corpseReviveChance*100)
            form.slider(`Corpses Despawn time in minutes`,1,360,1,corpseDespawnTime/60)
            form.toggle(`Notify dead player thier corpse despawns`,corpseTellWhenDespawned)
       
          
            sender.sendMessage(`${NOTIFICATION_PREFIX} Close the chat to open the settings menu`)

            forceShow(sender,form).then( ({canceled,formValues}) => {
                if(!canceled) {
                    world.setDynamicProperty(`corpseShowDeathCordinates`,formValues[0])
                    world.setDynamicProperty(`corpseAnyPlayerCanAccess`,formValues[1])
                    world.setDynamicProperty(`corpseAlwaysShowNameTag`,formValues[2])
                    world.setDynamicProperty(`corpseFloatsOnLiquids`,formValues[3])
                    world.setDynamicProperty(`corpseKeepXpOnDeath`,formValues[4])
                    world.setDynamicProperty(`corpseReviveChance`,formValues[5]/100)
                    world.setDynamicProperty(`corpseDespawnTime`,formValues[6]*60)
                    world.setDynamicProperty(`corpseTellWhenDespawned`,formValues[7])

                }
            })

                    
            
        }else{
            sender.sendMessage(noOpMessage)
        }
    }

    if ((message.substring(0,testCorpseCommand.length) == testCorpseCommand) && sender.isOp()){
        if(sender.isOp()){
            const {dimension,location} = sender
            const playerName = message.replace(testCorpseCommand,``)
        
            const tumba = dimension.spawnEntity(`ax:tumba`,location)
            const tEquipment = tumba.getComponent("equipment_inventory")
            const sword =  new server.ItemStack("stone_sword")
            tEquipment.setEquipment(`mainhand`,sword)

            if(world.getDynamicProperty(`corpseFloatsOnLiquids`)){
                tumba.triggerEvent(`float`)
            }
        
            if(world.getDynamicProperty(`corpseAlwaysShowNameTag`)){
                tumba.triggerEvent(`always_show_name`)
            }else{
                tumba.triggerEvent(`not_always_show_name`)
            }

            tumba.nameTag = CORPSE_NAME_PREFIX+playerName
            let corpseRemainingDespawnTime =  world.scoreboard.getObjective("corpseRemainingDespawnTime")
            corpseRemainingDespawnTime.setScore(tumba,world.getDynamicProperty(`corpseDespawnTime`))
            sender.sendMessage(`Corpse for hypothetical player with name \"${playerName}\" summoned`)
        }else{
            sender.sendMessage(noOpMessage)
        }

    }
    if ((message.substring(0,corpseDespawnCommand.length) == corpseDespawnCommand) && sender.isOp()){
        if(sender.isOp()){
            const {dimension,location} = sender
            const value = message.replace(corpseDespawnCommand,``)

            if(value === `true`){
                world.setDynamicProperty(`doCorpseDespawn`,true)
                sender.sendMessage(`Corpses will despawn`)
            }else if(value == `false`){
                world.setDynamicProperty(`doCorpseDespawn`,false)
                sender.sendMessage(`Corpses will not despawn`)
            }else{
                sender.sendMessage(`§c\"${value}\" is not a valid value, must be \"true\" or \"false\"`)
            }
            
        
            
        }else{
            sender.sendMessage(noOpMessage)
        }

    }

})


world.afterEvents.playerSpawn.subscribe(data => {
    
    //if ((data.entity) && data.entity.typeId == `minecraft:player`){
      data.player.runCommand(`camera @s clear`)
    //}
  })

world.afterEvents.entityDie.subscribe(({damageSource,deadEntity})=>{

    deadEntity.runCommand(`camera @s set minecraft:free ease 4 out_back pos ~ ~1.5 ~ facing ~~~`)

  
    const slots = ["offhand","mainhand","head","chest","legs","feet"]
    const {dimension, location } = deadEntity
    deadEntity.teleport({x: location.x,y: 900,z:location.z})
    const pInventory = deadEntity.getComponent(`inventory`).container
    const pEquipment = deadEntity.getComponent("equipment_inventory")
    let hasEquipment = false

    const rot = deadEntity.getRotation()
    const vel = deadEntity.getVelocity()


    const tumba = dimension.spawnEntity(`ax:tumba`,location)
    const tInventory = tumba.getComponent(`inventory`).container
    
    const tEquipment = tumba.getComponent("equipment_inventory")
    const air =  new server.ItemStack("air")

    if(!(world.getDynamicProperty(`corpseKeepXpOnDeath`))){
        deadEntity.runCommandAsync(`/xp -${deadEntity.level}L @s`)
    }
   
    slots.forEach(slot=>{
        let item = pEquipment.getEquipment(slot)
        if(item && !item.keepOnDeath){
           
            tEquipment.setEquipment(slot,item)
            pEquipment.setEquipment(slot,air)
            hasEquipment = true
            
        }
    })
    
    for (let i = 0; i < pInventory.size; i++) {
        let item = pInventory.getItem(i)
        
        if(item && !item.keepOnDeath){
          pInventory.setItem(i,air)
          tInventory.addItem(item)
          hasEquipment=true
        }
      }
    
    const {x,y,z} = location

    if(world.getDynamicProperty(`corpseShowDeathCordinates`)){
        deadEntity.sendMessage(`${NOTIFICATION_PREFIX} You died at ${Math.floor(x)} ${Math.floor(y)} ${Math.floor(z)}`)
    }
    let corpseRemainingDespawnTime =  world.scoreboard.getObjective("corpseRemainingDespawnTime")
    if (!hasEquipment){
        //tumba.triggerEvent(`despawn`)
        deadEntity.sendMessage(`${NOTIFICATION_PREFIX} You didn't had anything to drop so your corpse will despawn soon`)
        corpseRemainingDespawnTime.setScore(tumba,5)
    }else{
        if(world.getDynamicProperty(`corpseFloatsOnLiquids`)){
            tumba.triggerEvent(`float`)
        }
        if(world.getDynamicProperty(`corpseAlwaysShowNameTag`)){
            tumba.triggerEvent(`always_show_name`)
        }else{
            tumba.triggerEvent(`not_always_show_name`)
        }

        tumba.nameTag = CORPSE_NAME_PREFIX+deadEntity.name
        
        corpseRemainingDespawnTime.setScore(tumba,world.getDynamicProperty(`corpseDespawnTime`))

        tumba.setRotation(rot)
        tumba.applyImpulse(vel)
        if(!world.getDynamicProperty(`doCorpseDespawn`)){
            deadEntity.sendMessage(`${NOTIFICATION_PREFIX} Your corpse will despawn in ${world.getDynamicProperty(`corpseDespawnTime`)/60} minute(s)`)
        }
    }


    
}, {entityTypes: [`minecraft:player`]})




system.runInterval(()=>{
    let despawnTime =  world.scoreboard.getObjective("corpseRemainingDespawnTime")
    despawnTime.getParticipants().forEach(player =>{
        let d = despawnTime.getScore(player)
      if(d > 0){
        //if(player.hasTag("despawnTime")){
          //player.runCommandAsync("/scoreboard players remove @s despawnTime 1")
          
          despawnTime.setScore(player,d-1)
          
         
        }else if (d===0 && world.getDynamicProperty(`doCorpseDespawn`)){
            
            const p = world.getPlayers({name: player.getEntity().nameTag.replace(CORPSE_NAME_PREFIX,``)})[0]
            if(p && world.getDynamicProperty(`corpseTellWhenDespawned`)){
                p.sendMessage(`${NOTIFICATION_PREFIX} Your corpse has despawned`)
            }
            player.getEntity().triggerEvent(`despawn`)
        }
    })
    
  },20)




    




world.afterEvents.entityHitEntity.subscribe(  result => {
        const {hitEntity,damagingEntity} = result
        if ( (hitEntity.typeId == `ax:tumba`) && hitEntity.hasComponent(`minecraft:is_stunned`)) {
            const playerName = hitEntity.nameTag.replace(CORPSE_NAME_PREFIX,``)
            
            if(world.getDynamicProperty(`corpseAnyPlayerCanAccess`) || (damagingEntity.name === playerName)){
                if((world.getDynamicProperty(`corpseReviveChance`) != 0) && (Math.random()<=world.getDynamicProperty(`corpseReviveChance`)) && !hitEntity.hasComponent(`minecraft:is_chested`)){
                    hitEntity.triggerEvent(`waking_up`)
    
                    let corpseRemainingDespawnTime =  world.scoreboard.getObjective("corpseRemainingDespawnTime")
                    corpseRemainingDespawnTime.removeParticipant(hitEntity)
                }else{
                    corpseTransferItems(damagingEntity,hitEntity)
                }
            }else{
                damagingEntity.sendMessage(`${NOTIFICATION_PREFIX} Due to this world settings you can't acces other players' corpses`)
            }
          
          
        }
      
},{entityTypes:['minecraft:player']})
